# A popup window for choicing directories
#
# returns a list with directories
#
# Arguments:
# -initialdir "<start in dir> | home-dir (default)"
# -multiple "0 (default) | 1 (selecting of multiple dirs allowed)"
# -view_hidden_dirs "0 (default) | 1"
# -view_hidden_files "0 (default) | 1"
# -title "<your window title> | "Please choose a directory:" (default)"
# -parent "<window which starts FR_dirBox> | . (default)"
# -permission "writable | readable (default)"
# -balloonhelp "0 (default) | 1"
# -filetype "[list "Description" "*.file extension"] | * (default)"

package provide FR_dirBox 0.3

# we need some other packages
foreach Arg [list FR_messageBox FR_popupWindow FR_general FR_balloonHelp] {
 package require $Arg
}


################################################################### 
# Create the new namespace
###################################################################
namespace eval ::FR_dirBox {
 variable VAR
 
 namespace export Create
}


###################################################################
# The content of a directory
###################################################################
proc ::FR_dirBox::DirTreeContent {Path Dir} {
  variable VAR
 
  # move to the directory
  cd [lindex $Dir 0]
  # look for the "normal" directories 
  set Content [glob -types {d} -nocomplain *]
 
  if {$VAR($Path,HiddenDirs)} {
    # look for the hidden directories
	set Tmp [glob -types {d hidden} -nocomplain *]
    # the first two items in the search result are "." and ".."
    eval lappend Content [lrange $Tmp 2 end]
  }
 
  set Content [lsort -dictionary $Content]
   
  # if we not on the top directory we add the "go back" sign
  if { [lindex $Dir 0] != "/" } { set Content [linsert $Content 0 ".."] }
   
  return $Content
}


###################################################################
# Fills the directories into the listbox
###################################################################
proc ::FR_dirBox::DirTreeView {Path Dir} {   
  variable VAR
 
  set Content [::FR_dirBox::DirTreeContent $Path $Dir]
  # we clear the old stuff   
  .$VAR($Path,dirtree).dirtree delete 0 end
   
  foreach Arg $Content {
    # and fill in all new items
    .$VAR($Path,dirtree).dirtree insert end $Arg
  }
}


###################################################################
# Find the file type of the combobox and return it with glob 
# pattern
###################################################################
proc ::FR_dirBox::ShortFind {Arg} {
  
  # we are looking for *.(characters or numbers or *)
  set Result [regexp -all -inline -- {\*.([A-Za-z0-9\*]+)} $Arg]
  
  # we sort out the search return with the whole line
  set ShortFound ""
  for {set i 0} {$i < [llength $Result]} {incr i 2} {
    lappend ShortFound [lindex $Result $i]
  }
  
  # we replace e.g *.wav with *.[Ww][Aa][Vv]
  set Result ""
  foreach Arg $ShortFound {
    set NewArg ""
   
    # in case of *.* we set it to * to fetch files without tail also (e.g. Readme)
    if {$Arg == "*.*"} {
      set NewArg "*"
    } else {
      for {set j 0} {$j < [string length $Arg]} {incr j} {
        set Letter [string index $Arg $j]
    
        if {[string is lower $Letter]} {
          set Letter2 [string toupper $Letter]
          set NewLetter "\[${Letter2}${Letter}\]"
          append NewArg $NewLetter 
        } elseif {[string is upper $Letter]} {
          set Letter2 [string tolower $Letter]
          set NewLetter "\[${Letter}${Letter2}\]"
          append NewArg $NewLetter 
        } else {
          append NewArg $Letter
        }
      }
    }
   
    lappend Result $NewArg
  }
  
   return $Result 
} 


###################################################################
# Returns the files in "Dir"
###################################################################
proc ::FR_dirBox::FileTreeContent {Path Dir} {
  variable VAR
  
  set FileType [::FR_dirBox::ShortFind $VAR($Path,FileType)]
  
  set Content ""
  
  cd [lindex $Dir 0] 
  
  foreach Arg $FileType {
    set GlobReturnValue [glob -types {f} -nocomplain $Arg]
    
    if {$GlobReturnValue != ""} {
      eval lappend Content $GlobReturnValue
    }
  }
  
  if {$VAR($Path,HiddenFiles)} {
    foreach Arg $FileType {
      set GlobReturnValue [glob -types {hidden f} -nocomplain $Arg]
   
      if {$GlobReturnValue != ""} {
        eval lappend Content $GlobReturnValue
      }
    }
  }
  
  set Content [lsort -dictionary $Content]
 
  return $Content 
}


###################################################################
# Fills the files into the listbox
###################################################################
proc ::FR_dirBox::FileTreeView {Path Dir} {   
  variable VAR
   
  set Content [::FR_dirBox::FileTreeContent $Path $Dir] 
  
  .$VAR($Path,files).files delete 0 end  
  
  foreach Arg $Content {
    .$VAR($Path,files).files insert end $Arg
  } 
}


###################################################################
# Fills the chosen element into the entry-box
###################################################################
proc ::FR_dirBox::InsertText {Path Dir} {
  variable VAR
 
  .$VAR($Path,entry).entry configure -state normal 
  .$VAR($Path,entry).entry delete 0 end 
  .$VAR($Path,entry).entry insert end $Dir 
  .$VAR($Path,entry).entry configure -state disabled
}


###################################################################
# Refresh the view
###################################################################
proc ::FR_dirBox::DirTreeRefresh {Path {Dir ""}} {
  variable VAR
 
  # check if we should show a new directory
  if {[lindex $Dir 0] != ""} { 
    # save this new directory
	set VAR($Path,TmpDir) $Dir
  }
 
  ::FR_dirBox::InsertText $Path $VAR($Path,TmpDir)
    
  ::FR_dirBox::DirTreeView $Path $VAR($Path,TmpDir)
  ::FR_dirBox::FileTreeView $Path $VAR($Path,TmpDir)
} 


###################################################################
# The reaction of the double click on a directory
###################################################################
proc ::FR_dirBox::DirTreeSelection {Path Widget} { 
  variable VAR
 
  # fetch the index of the listbox
  set Index [$Widget curselection]
  if {$Index == ""} {return}
    
  # we use list here to allow directories with spaces
  set Selected [list [$Widget get $Index]]
   
  # check if the selected directory was the "go back" sign
  if { [lindex $Selected 0] == ".." } {
    # find the last / in the name to be eable to cut the following part 
    set LastSlash [string last / [lindex $VAR($Path,TmpDir) 0]]
     
    # to avoid the cutting of the leading / of the root-directory
    if { $LastSlash > 1} {
      # we go one character back because we don't the / anymore
      incr LastSlash -1
      # generate the new, shorter directory name
      set VAR($Path,TmpDir) [list [string range [lindex $VAR($Path,TmpDir) 0] 0 $LastSlash]] 
    } 
  } else {
    # the normal case: we add the selected directory to the TmpDir 
    set VAR($Path,TmpDir) [list [file join [lindex $VAR($Path,TmpDir) 0] [lindex $Selected 0]]]
  }
  
  ::FR_dirBox::DirTreeRefresh $Path $VAR($Path,TmpDir)
}
 
 
###################################################################
# We check the needed permissions for this directory
###################################################################
proc ::FR_dirBox::RightsCheck {Path Dir} {
  variable VAR
  
  if {$VAR($Path,Permission) == "writable"} { 
    set Cmd [list file writable $Dir]
    set Text [msgcat::mc "write-"]
  } else {
    set Cmd [list file readable $Dir]
    set Text [msgcat::mc "read-"]
  }
 
  if {![eval $Cmd]} {
    ::FR_messageBox::Create -type Error -parent .$Path \
	-message [msgcat::mc "You need the %spermission for this directory, but you havn't it." $Text]     
    return 0
  } 
    
  return 1 
}
 
 
###################################################################
# We read out the entry widget and return its content
###################################################################
proc ::FR_dirBox::EntryCheck {Path} {
  variable VAR
   
  set TestedDirs ""

  set DirList [.$VAR($Path,entry).entry get] 

  foreach Arg $DirList {
	# check the rights for each directory
    if {[::FR_dirBox::RightsCheck $Path $Arg]} {
      lappend TestedDirs $Arg
    }
  }
   
  if {[llength $TestedDirs] > 0} {
    set VAR($Path,SelectedDir) $TestedDirs
    focus $VAR($Path,Parent)
    destroy .$Path
    set VAR($Path,Stop) 1
  }
}


###################################################################
# Create a new directory
###################################################################
proc ::FR_dirBox::NewDir {Path} {
  variable VAR
  
  set Dir [::FR_popupWindow::Create -parent .$Path -description [msgcat::mc "Input your new directory:"]] 
  
  if {$Dir == ""} {return 1}
  
  set TmpDir [list [file join [lindex $VAR($Path,TmpDir) 0] [lindex $Dir 0]]] 

  if {[file exists [lindex $TmpDir 0]]} {
    ::FR_messageBox::Create -type Error -parent .$Path -message [msgcat::mc "This directory already exists."]  
    ::FR_dirBox::DirTreeRefresh $Path $TmpDir
    return 1
  }
   
  if { [catch { file mkdir [lindex $TmpDir 0] } ReturnContent] == 0 } {
    # refresh the directory window including the new directory
	::FR_dirBox::DirTreeRefresh $Path $TmpDir
  } else {
    ::FR_messageBox::Create -type Error -parent .$Path \
    -message [msgcat::mc "I can't create this directory!\nThe reason is:\n\n%s" $ReturnContent]
    ::FR_dirBox::DirTreeRefresh $Path
  }  
}	
 
 
###################################################################
# Change directly in another directory
###################################################################
proc ::FR_dirBox::ChangeDir {Path} {
  variable VAR
  
  set Dir [::FR_popupWindow::Create -parent .$Path -initialinput [lindex $VAR($Path,TmpDir) 0] \
  -description [msgcat::mc "Input your target directory:"]]
  
  if {$Dir == ""} {return 1}

  # check the wished directory
  if {[file isdirectory [lindex $Dir 0]]} {
    ::FR_dirBox::DirTreeRefresh $Path $Dir
  } else {
    ::FR_messageBox::Create -type Error -parent .$Path \
	-message [msgcat::mc "%s isn't a valid directory" [lindex $Dir 0]]
  }
}

 
###################################################################
# Delete directories
###################################################################
proc ::FR_dirBox::DeleteDir {Path} {
  variable VAR
   
  set Index [.$VAR($Path,dirtree).dirtree curselection]
  
  if {$Index == ""} {
    ::FR_messageBox::Create -type Error -parent .$Path \
	-message [msgcat::mc "Please select first a directory."]
   
    return 1
  }
  
  set Selected ""
     
  foreach i $Index {
    lappend Selected [file join [lindex $VAR($Path,TmpDir) 0] [.$VAR($Path,dirtree).dirtree get $i]]
  }
   
  set Answer [::FR_messageBox::Create -title [msgcat::mc "Are you sure?"] -type Info -parent .$Path \
  -message [msgcat::mc "Are you sure to delete the follow directories:\n\n%s" $Selected] \
  -button [list [msgcat::mc "Yes"] [msgcat::mc "No"]]] 
   
  if { $Answer } {
    # we start to delete
	foreach i $Selected {
      if { [catch { file delete -force $i } ReturnContent] } {
        ::FR_messageBox::Create -type Error -parent .$Path \
		-message [msgcat::mc "I can't delete this directory!\nThe reason is:\n\n%s" $ReturnContent]
      } 
    }
    
    ::FR_dirBox::DirTreeRefresh $Path
  }
}   


###################################################################
# Refresh the entry box after selecting
###################################################################
proc ::FR_dirBox::RefreshEntry {Path Widget} {
  variable VAR
  
  set Selected ""
  
  set Index [$Widget curselection]
  # check if there is anything selected
  if {$Index == ""} {return}
  
  # check if also the "go back" sign is selected
  if { [lindex $Index 0] == 0 } {
	# if it is selected -> we remove it from the list
    if {[$Widget get 0] == ".." } {set Index [lrange $Index 1 end]}
  }
   
  foreach i $Index {
    # generate a list with the full directory name
    lappend Selected [file join [lindex $VAR($Path,TmpDir) 0] [$Widget get $i]]
  }
  
  if {[llength $Selected] > 0} {::FR_dirBox::InsertText $Path $Selected}
}   


###################################################################
# The reaction after pressing the cancel button
###################################################################
proc ::FR_dirBox::CancelButton {Path} {
  variable VAR
  
  focus $VAR($Path,Parent)
  destroy .$Path
  set VAR($Path,SelectedDir) ""
  set VAR($Path,Stop) 1
}


###################################################################
# The ballonhelp
###################################################################
proc ::FR_dirBox::BalloonHelp {Path} {
  variable VAR
 
  ::FR_balloonHelp::AddHelp ".$VAR($Path,dirtree).dirtree" \
  [msgcat::mc "With a right-mouse-button-klick you get a menu with some directory functions"]
  
  ::FR_balloonHelp::AddHelp ".$VAR($Path,entry).entry" \
  [msgcat::mc "This is the list with your chosen directories"] 

  if {$VAR($Path,BalloonHelp)} {::FR_balloonHelp::Activate}
}


################################################################### 
# The main procedure
################################################################### 
proc ::FR_dirBox::Create {args} {
  global env
  variable VAR

  # first we generate a toplevel name who doesn't exist yet
  set i 1
  while {[winfo exists .filebox_dialog$i]} {incr i}
  set Path "filebox_dialog$i"
 
  # some presettings    
  set VAR($Path,Stop) 0
  set VAR($Path,TmpDir) [list $env(HOME)]
  set VAR($Path,HiddenDirs) 0
  set VAR($Path,HiddenFiles) 0
  set VAR($Path,Title) [msgcat::mc "Please choose a directory:"]
  set VAR($Path,Parent) .
  set VAR($Path,Permission) "readable" 
  set VAR($Path,Multiple) 0
  set VAR($Path,FileType) ""
  set VAR($Path,BalloonHelp) 0
  set VAR($Path,WorkingDir) [list [pwd]]
  
  # parse the arguments
  for {set i 0} {$i < [llength $args]} {incr i 2} {
    set Arg [lindex $args $i]
    switch -glob -- $Arg {
      -initialdir        {set VAR($Path,TmpDir) [lindex $args [expr {$i+1}]]} \
      -view_hidden_files {set VAR($Path,HiddenFiles) [lindex $args [expr {$i+1}]]} \
      -view_hidden_dirs  {set VAR($Path,HiddenDirs) [lindex $args [expr {$i+1}]]} \
      -title             {set VAR($Path,Title) [lindex $args [expr {$i+1}]]} \
      -parent            {set VAR($Path,Parent) [lindex $args [expr {$i+1}]]} \
      -permission        {set VAR($Path,Permission) [lindex $args [expr {$i+1}]]} \
      -multiple          {set VAR($Path,Multiple) [lindex $args [expr {$i+1}]]} \
      -filetype          {set VAR($Path,FileType) [lindex $args [expr {$i+1}]]} \
      -balloonhelp       {set VAR($Path,BalloonHelp) [lindex $args [expr {$i+1}]]} \
      --                 {break} \
      *                  {puts "Unknown option: $Arg"; exit 1}
    }
  }

  # we generate a new toplevel window
  toplevel .$Path
 
  # we set the windowsname and hide it
  wm title .$Path "$VAR($Path,Title)"
  wm withdraw .$Path
  
  # we check the arguments
  # check if a wrong directory was handed over... 
  if {![file isdirectory [lindex $VAR($Path,TmpDir) 0]]} {
    # ...we set it to HOME 
    set VAR($Path,TmpDir) [list $env(HOME)]
  }
 
  # if no file type was set...
  if {$VAR($Path,FileType) == ""} {
    # ...we set it to "All files" 
    set VAR($Path,FileType) [list [msgcat::mc "All files"] "*.*"]
  }
 
  # if no parent window was set...
  if {![winfo exists $VAR($Path,Parent)]} {
	# ...we set it to "."
    set VAR($Path,Parent) .
  }

  # we set the selectmode of the file
  if {$VAR($Path,Multiple)} {
    set VAR($Path,ListBoxMode) "extended"
  } else {
    set VAR($Path,ListBoxMode) "browse"
  }
     
  # the frame for the buttons
  frame .$Path.frame2 -relief flat
  frame .$Path.frame3 -relief groove
  frame .$Path.frame4 -relief groove 
  frame .$Path.frame5 -relief groove 
 
  pack  .$Path.frame2 -side bottom -fill x
  pack  .$Path.frame3 -side bottom -fill x -padx 5 -pady 5
  pack  .$Path.frame4 -side left -padx 5 -pady 5 -expand yes -fill both
  pack  .$Path.frame5 -side left -padx 5 -pady 5 -expand yes -fill both

  
  # We set a variable for each frame - so we can change the order of the frames later easier
  set VAR($Path,botton) "$Path.frame2"
  set VAR($Path,entry) "$Path.frame3"
  set VAR($Path,dirtree) "$Path.frame4"
  set VAR($Path,files) "$Path.frame5"
 
  label .$VAR($Path,dirtree).desc -text [msgcat::mc "Directory tree:"]
  listbox .$VAR($Path,dirtree).dirtree -background white -selectmode $VAR($Path,ListBoxMode) \
  -yscroll ".$VAR($Path,dirtree).yscrollbar set" -xscroll ".$VAR($Path,dirtree).xscrollbar set"
 
  scrollbar .$VAR($Path,dirtree).yscrollbar -orient vertical -command ".$VAR($Path,dirtree).dirtree yview"
  scrollbar .$VAR($Path,dirtree).xscrollbar -orient horizontal -command ".$VAR($Path,dirtree).dirtree xview"
 
  grid .$VAR($Path,dirtree).desc -row 0 -column 0 -sticky w 
  grid .$VAR($Path,dirtree).dirtree -row 1 -column 0 -sticky nsew 
  grid .$VAR($Path,dirtree).yscrollbar -row 1 -column 1 -sticky ns 
  grid .$VAR($Path,dirtree).xscrollbar -row 2 -column 0 -sticky ew 

  grid columnconfigure .$VAR($Path,dirtree) 0 -weight 1
  grid rowconfigure .$VAR($Path,dirtree) 1 -weight 1

  # a menu to delete or create new dirs
  menu .$VAR($Path,dirtree).menu -tearoff 0
  .$VAR($Path,dirtree).menu add command -label [msgcat::mc "Go to home directory"] \
  -command [list ::FR_dirBox::DirTreeRefresh $Path [list $env(HOME)]]
  
  .$VAR($Path,dirtree).menu add separator
 
  .$VAR($Path,dirtree).menu add command -label [msgcat::mc "Go to / -directory"] \
  -command [list ::FR_dirBox::DirTreeRefresh $Path [list "/"]]
 
  .$VAR($Path,dirtree).menu add separator 
  
  .$VAR($Path,dirtree).menu add command -label [msgcat::mc "Go directly into another directory"] \
  -command [list ::FR_dirBox::ChangeDir $Path] 
  
  .$VAR($Path,dirtree).menu add separator 
  
  .$VAR($Path,dirtree).menu add command -label [msgcat::mc "Add a new directory"] \
  -command [list ::FR_dirBox::NewDir $Path]
  
  .$VAR($Path,dirtree).menu add separator
  
  .$VAR($Path,dirtree).menu add command -label [msgcat::mc "Delete the selected directory"] \
  -command [list ::FR_dirBox::DeleteDir $Path] 

  bind .$VAR($Path,dirtree).dirtree <<ListboxSelect>> [list ::FR_dirBox::RefreshEntry $Path %W]   
  bind .$VAR($Path,dirtree).dirtree <Double-Button-1> [list ::FR_dirBox::DirTreeSelection $Path %W]
  bind .$VAR($Path,dirtree).dirtree <Button-3> [list tk_popup .$VAR($Path,dirtree).menu %X %Y]
 
  # the listbox for the files
  label .$VAR($Path,files).desc -text [msgcat::mc "Files:"] 
  listbox .$VAR($Path,files).files -background white \
  -yscroll ".$VAR($Path,files).yscrollbar set" -xscroll ".$VAR($Path,files).xscrollbar set"
 
  scrollbar .$VAR($Path,files).yscrollbar -orient vertical -command ".$VAR($Path,files).files yview"
  scrollbar .$VAR($Path,files).xscrollbar -orient horizontal -command ".$VAR($Path,files).files xview"
 
  grid .$VAR($Path,files).desc -row 0 -column 0 -sticky w
  grid .$VAR($Path,files).files -row 1 -column 0 -sticky nsew 
  grid .$VAR($Path,files).yscrollbar -row 1 -column 1 -sticky ns
  grid .$VAR($Path,files).xscrollbar -row 2 -column 0 -sticky ew

  grid columnconfigure .$VAR($Path,files) 0 -weight 1
  grid rowconfigure .$VAR($Path,files) 1 -weight 1 

  # The ok and chancel buttons
  set ButtonText [::FR_general::ButtonsWithSameWidth [list [msgcat::mc "Ok"] [msgcat::mc "Cancel"]]]
 
  button .$VAR($Path,botton).ok -text [lindex $ButtonText 0] -state normal \
  -width [lindex $ButtonText 2] -command [list ::FR_dirBox::EntryCheck $Path]
  button .$VAR($Path,botton).cancel -text [lindex $ButtonText 1] -state normal \
  -width [lindex $ButtonText 2] -command [list ::FR_dirBox::CancelButton $Path]
  
  pack .$VAR($Path,botton).ok -side left -padx 5 -pady 5 -fill x -expand yes
  pack .$VAR($Path,botton).cancel -side left -padx 5 -pady 5 -fill x -expand yes
 
  bind .$Path <KeyPress-Escape> [list .$VAR($Path,botton).cancel invoke]
  bind .$Path <KeyPress-Return> [list .$VAR($Path,botton).ok invoke]
 
  # the entry-box (no input only for controlling)
  label .$VAR($Path,entry).desc -text [msgcat::mc "Chosen directory:"]
  entry .$VAR($Path,entry).entry -state disable -xscrollcommand ".$VAR($Path,entry).mkdir_xsbar set" 
  scrollbar .$VAR($Path,entry).mkdir_xsbar -orient horizontal -command ".$VAR($Path,entry).entry xview"
  
  grid .$VAR($Path,entry).desc -row 0 -column 0 -sticky w -padx 5
  grid .$VAR($Path,entry).entry -row 1 -column 0 -sticky ew -padx 5
  grid .$VAR($Path,entry).mkdir_xsbar -row 2 -column 0 -sticky nsew -padx 5
  
  grid columnconfigure .$VAR($Path,entry) 0 -weight 1
 
  # fill all list boxes
  ::FR_dirBox::DirTreeRefresh $Path $VAR($Path,TmpDir)
 
  eval ::FR_general::CenterWindow .$Path $VAR($Path,Parent) 1 450 300
  wm deiconify .$Path
  focus .$Path
 
  ::FR_dirBox::BalloonHelp $Path

  wm protocol .$Path WM_DELETE_WINDOW [list .$VAR($Path,botton).cancel invoke]
 
  # We stop the procedure and wait until the return value is ready
  vwait ::FR_dirBox::VAR($Path,Stop)
  return $VAR($Path,SelectedDir)
}

