#---FR_exec based of a code of Frank Pilhofer
#---and execlog.tcl from Brent B. Welch

# usage:
# ::FR_exec::Start Command {Time in ms | default: infinity} {Kill the process after finishing | default: 0}
#
# FR_exec opens a pipe for "Command"
# if there no output from "Command" after "Time" we will break
#
# The return value is a list:
# [0 Output_of_the_background_tool] if the tool in the background was successfully
# [1 Error_reason] if not

package provide FR_exec 0.2

namespace eval ::FR_exec {
 namespace export Start
}

# ---------------------------------------------- 
# we open a channel
# ---------------------------------------------- 
proc ::FR_exec::Start {Command {Time "infinity"} {Kill 0}} {

  variable ExecId; variable ExecContinue; variable ExecChannel; variable ExecData;
  variable ExecTimer; variable ExecPid; variable ExecKill; variable ExecLastLine;
  variable ExecErrorMsg; 

  # check if FR_exec was already called 
  if { ![info exists ExecId] } {
	set ExecId 0
  } else {
	incr ExecId
  }

  # open the command a redirect the stderr ot stdout
  if { [catch {open "|$Command 2>@ stdout"} ChannelId] } {
	return [list 1 $ChannelId]
  }

  set Id $ExecId
  set ExecChannel($Id) $ChannelId
  set ExecPid($Id) [lindex [pid $ChannelId] 0]
  set ExecKill($Id) $Kill
  set ExecData($Id) ""
  set ExecContinue($Id) 0
  set ExecLastLine($Id) ""
  set ExecErrorMsg($Id) ""


  if { $Time == "infinity" } {
	set ExecTimer($Id) $Time
  } else {
	set ExecTimer($Id) [after $Time [list ::FR_exec::Break $Id]]
  }

  fconfigure $ChannelId -blocking 1
  fileevent $ChannelId readable [list ::FR_exec::FileEvent $Id]

  vwait ::FR_exec::ExecContinue($Id)

  unset ExecChannel($Id)
  unset ExecContinue($Id)

  if { $ExecErrorMsg($Id) == "" } {
  return [list 0 $ExecData($Id)]
  } else {
  return [list 1 $ExecErrorMsg($Id)] 
  }
}


# ---------------------------------------------- 
# if the channel readable we execute this
# ---------------------------------------------- 
proc ::FR_exec::FileEvent {Id} { 
 variable ExecChannel; variable ExecData; variable ExecLastLine;

 if { [eof $ExecChannel($Id)] } {
   ::FR_exec::Stop $Id
 } else {
   gets $ExecChannel($Id) Line
   
   # we don't save more than one empty line
   #if { ($Line == "") && ($ExecLastLine($Id) == "") } {
     # we skip this line
   #} else {
     append ExecData($Id) $Line\n
   #  set ExecLastLine($Id) $Line
   #}
 }
}


# ---------------------------------------------- 
# we break after "Time"
# ---------------------------------------------- 
proc ::FR_exec::Break {Id} {
 variable ExecData
 
 set ExecData($Id) ""
 
 ::FR_exec::Stop $Id
}


# ---------------------------------------------- 
# the stop command after break or after EOF
# ---------------------------------------------- 
proc ::FR_exec::Stop {Id} {
	variable ExecContinue; variable ExecChannel; variable ExecTimer; variable ExecPid;
	variable ExecKill; variable ExecErrorMsg; 

	# stop the watchdog
	if { $ExecTimer($Id) != "infinity" } {
		after cancel $ExecTimer($Id)
 }

 # erase the event binding
 fileevent $ExecChannel($Id) readable ""

 if { [catch {close $ExecChannel($Id)} Error] } {
	 set ExecErrorMsg($Id) "Channel error: $Error"
 }

 if { $ExecKill($Id) } {
	 catch {exec kill $ExecPid($Id)}
 }

 set ExecContinue($Id) 1
}

