#---the text widget as message window
#
# ::FR_msgWindow::Create
# -window "a frame in which we can insert the message widget"
# -height (the height of the text widget (default: 10)
# 
# ::FR_msgWindow::Add
# -window (path to the msg-window) if no path was overtaken: default is the last created msg-window
# -format (Heading | Info | Error | Body | BodyRed) default: Body
# -msg (message)


package provide FR_msgWindow 0.3


namespace eval ::FR_msgWindow \
{
  namespace export Create Add
}


proc ::FR_msgWindow::Create {args} \
{
  variable VAR; variable PATH
 
  set Path ""
  set Height 10
 
  for {set i 0} {$i < [llength $args]} {incr i 2} \
  {
    set Arg [lindex $args $i]
    switch -glob -- $Arg \
    {
      -window  {set Path [lindex $args [expr {$i+1}]]} \
      -height  {set Height [lindex $args [expr {$i+1}]]} \
      --       {break} \
      *        {puts "Unknown option: $Arg"; exit 1}
    }
  }

   
  if { $Path == "" } \
  {
    puts "You must set a window name for your msgWindow with \"-window\""
    exit 1
  }
 
  set Path [string trimleft $Path .]
  set PATH $Path
 
 
  label .$Path.topic -text [msgcat::mc "Process messages:"]
  button .$Path.clear_button -text [msgcat::mc "Clear"] -command [list ::FR_msgWindow::Clear $Path.text]

  text .$Path.text -height $Height -wrap none -background white -state disabled -xscrollcommand ".$Path.xsbar set" -yscrollcommand ".$Path.ysbar set"
  scrollbar .$Path.xsbar -orient horizontal -command ".$Path.text xview"
  scrollbar .$Path.ysbar -orient vertical -command ".$Path.text yview"
 
  grid .$Path.topic -row 0 -column 0 -sticky w -pady 5
  grid .$Path.clear_button -row 0 -column 2 -columnspan 2 -sticky e -pady 5
  grid .$Path.text -row 1 -column 0 -columnspan 3 -sticky nsew
  grid .$Path.ysbar -row 1 -column 3 -sticky ns
  grid .$Path.xsbar -row 2 -column 0 -columnspan 3 -sticky ew
 
  grid columnconfigure .$Path 1 -weight 1
  grid rowconfigure .$Path 1 -weight 1
 
  # we configure the text tags
  .$Path.text tag configure "Heading" -foreground blue -underline yes -lmargin1 0.05i -spacing1 0.05i -spacing2 0
  .$Path.text tag configure "Info" -foreground black -lmargin1 0.05i -spacing1 0.05i -spacing2 0
  .$Path.text tag configure "Error" -foreground red -lmargin1 0.05i -spacing1 0.05i -spacing2 0
  .$Path.text tag configure "Body" -foreground black -lmargin1 0.1i
  .$Path.text tag configure "BodyRed" -foreground red -lmargin1 0.1i
}


#---add some informations to the msg window
proc ::FR_msgWindow::Add {args}\
{
  variable VAR; variable PATH
 
  set Window ""
  set Format ""
  set Message ""
 
  for {set i 0} {$i < [llength $args]} {incr i 2} \
  {
    set Arg [lindex $args $i]
    switch -glob -- $Arg \
    {
      -window  {set Window [lindex $args [expr {$i+1}]]} \
      -format  {set Format [lindex $args [expr {$i+1}]]} \
      -msg     {set Message [lindex $args [expr {$i+1}]]} \
      --       {break} \
      *        {puts "Unknown option: $Arg"; exit 1}
    }
  }
 
  # if now path was overtaken we assume that you have only one msg window
  # or you mean the msg window which was created recently
  if {$Window == ""} {set Window $PATH}
 
  if {$Format == ""} {set Format "Body"}
 
  .$Window.text configure -state normal
  
  .$Window.text insert end $Message $Format "\n"
  
  .$Window.text see end
  
  update
  
  .$Window.text configure -state disabled
}


#---we clear the msg window
proc ::FR_msgWindow::Clear {Path} \
{
  .$Path configure -state normal
  .$Path delete 1.0 end
  
  #update
  
  .$Path configure -state disabled
}
