#---A notebook widget based on rnotebook of Daniel Roche and Earl Roosa
# 
# Copyright (C) 1999 Daniel Roche
#               2008 Falk Richter
#
# usage:
# 
# ::FR_notebook::Create widget_name [option value ...]
# ---------------------
# Creates a new notebook widget
#  options:
#   [-borderwidth i]      set the borderwidth of the tabs and the frame (default: 2)
#   [-height i]           set the height of the tabs in pixel (default: 30)
#   [-padx i]             set the space between each tabs in pixel (default: 0)
#   [-background col]     set the background color for the tabs and the frame (default: standard)
#   [-foreground col]     set the forground color for the tabs (default: standard)
#   [-command i]          execute the command i after showing the tab
#
#
# ::FR_notebook::AddTab widget_name [option value ...]
# ---------------------
# Adds a new tab to the notebook widget
#  options:
#   [-tab i]              the name of the tab
#   [-text i]             showed text for the tab (default: the same as tab)
#   [-extreturn i]        if true returns frame and button path (default: returns frame path)
#
#
# ::FR_notebook::DeleteTab widget_name [option value ...]
# ------------------------
# Delets a tab of the notebook widget
#  options:
#   [-tab i]              the name of the tab
#
#
# ::FR_notebook::CurrentTab widget_name
# -------------------------
# Returns the current raised tab
#
#
# ::FR_notebook::GetAllTabs widget_name
# -------------------------
# Returns a list with all tabs
#
#
# ::FR_notebook::FramePath widget_name [option value ...]
# ------------------------
# Returns the path to the frame of tab
#  options:
#   [-tab i]              the name of the tab
#
# ::FR_notebook::CurrentFramePath widget_name
# ------------------------
# Returns the path to the frame of the current tab
#
#
# ::FR_notebook::ButtonPath widget_name [option value ...]
# ------------------------
# Returns the path to the button of tab
#  options:
#   [-tab i]              the name of the tab
#
#
# ::FR_notebook::Raise widget_name [option value ...]
# Show tab
#  options:
#   [-tab i]              the name of the tab (default: the current tab)
#

package provide FR_notebook 0.1

namespace eval ::FR_notebook \
{
 namespace export Create AddTab DeleteTab CurrentTab GetAllTabs FramePath CurrentFramePath ButtonPath Raise
}


#---Create a new notebook widget
proc ::FR_notebook::Create {args} \
{
 variable VAR;  
 
 # we read the pathname
 set Path ""
 set Path [lindex $args 0]

 # we check the correctness of the pathname
 if {$Path == ""} {error "You must hand over a valid pathname, not an empty string"}
  
 set Path [string trimleft $Path .]
 
 set args [lrange $args 1 end]
 
 # init base widget
 frame .$Path -borderwidth 0 -relief flat
 
 # init default data structure
 set VAR($Path,Tabs) ""
 set VAR($Path,FrameIndex) 0
 set VAR($Path,Current) ""
 set VAR($Path,CurrentIndex) ""
 set VAR($Path,CallBackPending) ""
 set VAR($Path,TabHeight) 30
 set VAR($Path,PadX) 0
 set VAR($Path,BorderWidth) 1
 set VAR($Path,BackGround) [.$Path cget -background]
 set VAR($Path,ForeGround) black
 set VAR($Path,Command) ""
 
 # parse arguments
 foreach {Arg Value} $args \
 {
  switch -glob -- $Arg \
  {
   -borderwidth {set VAR($Path,BorderWidth) $Value} \
   -height      {set VAR($Path,TabHeight) $Value} \
   -padx        {set VAR($Path,PadX) $Value} \
   -background  {set VAR($Path,BackGround) $Value} \
   -foreground  {set VAR($Path,ForeGround) $Value} \
   -command     {set VAR($Path,Command) $Value} \
   --           {break} \
   *            {puts "Unknown option: $Arg"; exit 1} \
  }
 }
 
 # the tabs frame contains the buttons for selection
 frame .$Path.tabs -borderwidth 0 -relief flat
 # the body frame is the main frame for the content (each content has a separate frame)
 frame .$Path.body -borderwidth $VAR($Path,BorderWidth) -relief raised -background $VAR($Path,BackGround)
 
 # mask hides the bottom borderwidth of the current showed button (with exception of the corners)
 frame .$Path.mask -borderwidth 0 -relief flat -background $VAR($Path,BackGround)
 
 # mask_left hides the bottom borderwidth (left corner)
 frame .$Path.mask_left -borderwidth 0 -relief flat -background $VAR($Path,BackGround)
 frame .$Path.mask_left.ml -borderwidth $VAR($Path,BorderWidth) -relief sunken
 place .$Path.mask_left.ml -x -$VAR($Path,BorderWidth) -y -$VAR($Path,BorderWidth) -width [expr 2 * $VAR($Path,BorderWidth)] -height [expr 3 * $VAR($Path,BorderWidth)]
 
 # mask_right hides the bottom borderwidth (right corner)   
 frame .$Path.mask_right -borderwidth 0 -relief flat -background $VAR($Path,BackGround)
 frame .$Path.mask_right.mr -borderwidth $VAR($Path,BorderWidth) -relief sunken
 place .$Path.mask_right.mr -x 0 -y -$VAR($Path,BorderWidth) -width [expr 2 * $VAR($Path,BorderWidth)] -height [expr 3 * $VAR($Path,BorderWidth)]

 place .$Path.tabs -x 0 -y 0 -relwidth 1.0 -height $VAR($Path,TabHeight)
 place .$Path.body -x 0 -y $VAR($Path,TabHeight) -relwidth 1.0 -relheight 1.0 -height -$VAR($Path,TabHeight)
 
# button .$Path.tabs.menu -text ">" -borderwidth $VAR($Path,BorderWidth) -background $VAR($Path,BackGround) -relief raised \
# -foreground $VAR($Path,ForeGround) -command {} -highlightthickness 0
# 
# pack .$Path.tabs.menu -side right -pady 0 -padx $VAR($Path,PadX) -fill y
}


#---add a new tab
proc ::FR_notebook::AddTab {args} \
{
 variable VAR
 
 # we read the pathname
 set Path ""
 set Path [lindex $args 0]

 # we check the correctness of the pathname
 if {$Path == ""} {error "You must hand over a valid pathname, not an empty string"}
  
 set Path [string trimleft $Path .]
 
 set args [lrange $args 1 end]
 
 set Text ""
 set Tab ""
 set LongReturn 0
 
 # parse arguments
 foreach {Arg Value} $args \
 {
  switch -glob -- $Arg \
  {
   -text      {set Text $Value} \
   -tab       {set Tab $Value} \
   -extreturn {set LongReturn $Value} \
   --         {break} \
   *          {puts "Unknown option: $Arg"; exit 1} \
  }
 }
 
 if {$Tab == ""} {error "You must hand over a valid description for your tab, not an empty string"}
 
 # we create a list with the tab name and the index for the frame of the tab
 lappend VAR($Path,Tabs) $Tab $VAR($Path,FrameIndex) 
 
 if {$Text == ""} {set Text $Tab}

 set ButtonPath $Path.tabs.b$VAR($Path,FrameIndex)
 button .$ButtonPath -text $Text -borderwidth $VAR($Path,BorderWidth) -background $VAR($Path,BackGround) -relief raised \
 -foreground $VAR($Path,ForeGround) -command [list ::FR_notebook::RaiseSub $Path $VAR($Path,FrameIndex) $Tab] -highlightthickness 0
 
 pack .$ButtonPath -side left -pady 0 -padx $VAR($Path,PadX) -fill y
 bind .$ButtonPath <Configure> [list ::FR_notebook::ConfigureCallBack $Path $VAR($Path,FrameIndex)]
 
 # we save the name of the frame in "Returnvalue"
 set FramePath $Path.body.f$VAR($Path,FrameIndex)
 frame .$FramePath -borderwidth 0 -background $VAR($Path,BackGround) -highlightthickness 0
 
 
 ::FR_notebook::DrawBridge $Path $VAR($Path,FrameIndex)

 incr VAR($Path,FrameIndex)
 
 if {$LongReturn} \
 {
  return [list $FramePath $ButtonPath]
 } else \
 {
  return $FramePath
 }
}


#---delete a tab
proc ::FR_notebook::DeleteTab {args} \
{
 variable VAR
 
 # we parse the arguments
 set Args [eval ::FR_notebook::Arguments $args]
 set Path [lindex $Args 0]
 set Tab [lindex $Args 1]
 
 if {$Tab == ""} {error "You must hand over a valid description for your tab, not an empty string"}
 
 set Index [lsearch $VAR($Path,Tabs) $Tab]
 if {$Index == -1} \
 {
  puts "Error! Can not find: $Tab in $VAR($Path,Tabs)"
  return $Index
 }
 
 # because VAR($Path,Tabs) is always a pair of tab and frameindex
 set NextIndex [expr {$Index + 1}]
 
 set FrameIndex [lindex $VAR($Path,Tabs) $NextIndex]
 
 pack forget .$Path.body.f$FrameIndex
 pack forget .$Path.tabs.b$FrameIndex
 
 destroy .$Path.body.f$FrameIndex
 destroy .$Path.tabs.b$FrameIndex
 
 set VAR($Path,Tabs) [lreplace $VAR($Path,Tabs) $Index $NextIndex]
 
 if {[llength $VAR($Path,Tabs)] > 1} \
 {
  set Tab [lindex $VAR($Path,Tabs) end-1]
  set FrameIndex [lindex $VAR($Path,Tabs) end]
  
  ::FR_notebook::RaiseSub $Path $FrameIndex $Tab
 } \
 else \
 {
  set VAR($Path,Current) ""
  set VAR($Path,CurrentIndex) ""
  
  ::FR_notebook::DeleteBridge $Path
 }
}


#---give back the current visible tab
proc ::FR_notebook::CurrentTab {Path} \
{
 variable VAR
 
 set Path [string trimleft $Path .]
    
 return $VAR($Path,Current)
}


#---returns all created notebook tabs
proc ::FR_notebook::GetAllTabs {Path} \
{
 variable VAR
 
 set Path [string trimleft $Path .]
 
 set ReturnList ""
 
 foreach {Tab FrameIndex} $VAR($Path,Tabs) \
 {
  lappend ReturnList $Tab
 }
 
 return $ReturnList
}


# Return the frame associated with the tab
proc ::FR_notebook::FramePath {args} \
{
 variable VAR
 
 # we parse the arguments
 set Args [eval ::FR_notebook::Arguments $args]
 set Path [lindex $Args 0]
 set Tab [lindex $Args 1]
 
 set FrameIndex [::FR_notebook::FindTab $Path $Tab]
 if {$FrameIndex == -1} {return}
 
 return $Path.body.f$FrameIndex
}


# Return the frame associated with the current showed tab
proc ::FR_notebook::CurrentFramePath {Path} \
{
 variable VAR
 
 set Path [string trimleft $Path .]
 
 if {$VAR($Path,CurrentIndex) == ""} {return}
 
 return $Path.body.f$VAR($Path,CurrentIndex)
}


# Return the button associated with the tab
proc ::FR_notebook::ButtonPath {args} \
{
 variable VAR
 
 # we parse the arguments
 set Args [eval ::FR_notebook::Arguments $args]
 set Path [lindex $Args 0]
 set Tab [lindex $Args 1]
 
 set FrameIndex [::FR_notebook::FindTab $Path $Tab]
 if {$FrameIndex == -1} {return}
 
 return $Path.body.b$FrameIndex
}


#---show the tab frame
proc ::FR_notebook::Raise {args} \
{
 variable VAR
 
 # we parse the arguments
 set Args [eval ::FR_notebook::Arguments $args]
 set Path [lindex $Args 0]
 set Tab [lindex $Args 1]
 
 if {$Tab ne ""} \
 {
  set FrameIndex [::FR_notebook::FindTab $Path $Tab]
  if {$FrameIndex == -1} {return 1}
 } else \
 {
  # if there isn't a frame yet we can't set it automaticaly
  if {$VAR($Path,CurrentIndex) == ""} {return 1}
  
  set FrameIndex $VAR($Path,CurrentIndex)
  set Tab $VAR($Path,Current)
 }
 
 ::FR_notebook::RaiseSub $Path $FrameIndex $Tab
}


#---the real raising function
proc ::FR_notebook::RaiseSub {Path FrameIndex Tab} \
{
 variable VAR
 
 # we must only do anything if the wished frame isn't the current frame
 if {$FrameIndex != $VAR($Path,CurrentIndex)} \
 {
  # after adding the first tab there doesn't exist a current frame
  if {$VAR($Path,CurrentIndex) ne ""} \
  {
   pack forget .$Path.body.f$VAR($Path,CurrentIndex)
  }
  
  pack .$Path.body.f$FrameIndex -fill both -expand 1 -padx 10 -pady 10
  
  set VAR($Path,Current) $Tab
  set VAR($Path,CurrentIndex) $FrameIndex
 }
 
 ::FR_notebook::DrawBridge $Path $FrameIndex
 
 if {$VAR($Path,Command) ne ""} {eval $VAR($Path,Command)}
}


#---find the tab in the tablist
proc ::FR_notebook::FindTab {Path Tab} \
{
 variable VAR
 
 set Index [lsearch $VAR($Path,Tabs) $Tab]
 if {$Index == -1} \
 {
  puts "Error! Can not find: $Tab in $VAR($Path,Tabs)"
  return $Index
 }
 
 set FrameIndex [lindex $VAR($Path,Tabs) [expr {$Index + 1}]]
 return $FrameIndex
}


#---the callback for all notebook buttons
proc ::FR_notebook::ConfigureCallBack {Path FrameIndex} \
{
 variable VAR
 
 if {$FrameIndex != $VAR($Path,CurrentIndex)} {return}
 
 if {$VAR($Path,CallBackPending) == ""} \
 {
  set VAR($Path,CallBackPending) [after idle [list ::FR_notebook::DrawBridge $Path $FrameIndex]]
 }
}


#---connects the current button with the current frame
proc ::FR_notebook::DrawBridge {Path FrameIndex} \
{
 variable VAR
 
 set bw $VAR($Path,BorderWidth)

 set x0 [expr [winfo x .$Path.tabs.b$FrameIndex] + [winfo x .$Path.tabs] + $bw ]
 set y0 [expr [winfo y .$Path.tabs.b$FrameIndex] + [winfo y .$Path.tabs] + [winfo height .$Path.tabs.b$FrameIndex] - $bw]
 set w0 [expr [winfo width .$Path.tabs.b$FrameIndex] - ($bw * 2)]
 set h0 [expr $bw * 2]
 
 place .$Path.mask -x $x0 -y $y0 -width $w0 -height $h0

 set x1 [expr $x0 - $bw]
 set y1 $y0
 set w1 $bw
 set h1 $h0
 place .$Path.mask_left -x $x1 -y $y1 -width $w1 -height $h1
    
 set x2 [expr $x0 + $w0]
 set y2 $y0
 set w2 $bw
 set h2 $h0
 place .$Path.mask_right -x $x2 -y $y2 -width $w2 -height $h2
 
 set VAR($Path,CallBackPending) ""
}


#---if you delete the last tab we must remove the bridge
proc ::FR_notebook::DeleteBridge {Path} \
{
 foreach ARG [list $Path.mask $Path.mask_left $Path.mask_right] \
 {
  place forget .$ARG
 }
} 


#---we parse the standard arguments
proc ::FR_notebook::Arguments {args} \
{
 # we read the pathname
 set Path ""
 set Path [lindex $args 0]
 
 # we check the correctness of the pathname
 if {$Path == ""} {error "You must hand over a valid pathname, not an empty string"}
 set Path [string trimleft $Path .]
 
 set args [lrange $args 1 end]
 
 set Tab ""
 
 # parse arguments
 foreach {Arg Value} $args \
 {
  switch -glob -- $Arg \
  {
   -tab  {set Tab $Value} \
   --    {break} \
   *     {puts "Unknown option: $Arg"; exit 1} \
  }
 }
 
 return [list $Path $Tab]
}
