#
# a box to select multiple files / directories
#
# Copyright (C) 2009 Falk Richter
#
# ::FR_selectBox::Create
#
# -parent "<window which starts FR_selectBox> | . (default)"
# -title "<your window title> | "Files / directories selector" (default)"
# -type "Dirs | Files | Dirs_Files (default)"
# -initialdir "<start in dir> | home-dir (default)"
# -initialfile "<your file name proposal> | nothing (default)"
# -multiple "0 (default) | 1 (selecting of multiple files allowed)"
# -preload "<preload this files into the box with the chosen items> | nothing (default)"
# -filetype "[list "Description" "*.file extension"]"
# -only_filetype "0 add "All files" "*.*" to the -filedir (default) | 1 only types under -filetype"
# -view_hidden_files "0 (default) | 1"
# -view_hidden_dirs "0 (default) | 1"
# -permission "writable | readable (default)"
# -balloonhelp "0 (default) | 1"


package provide FR_selectBox 0.1

namespace eval ::FR_selectBox \
{
  namespace export Create
}


foreach Arg [list FR_general FR_fileBox FR_dirBox] \
{  
  package require $Arg
}


# the main procedure
proc ::FR_selectBox::Create {args} \
{
  global env
  variable VAR
 
  # first we generate a toplevel name who doesn't exist yet
  set i 1
  while {[winfo exists .selbox_window$i]} {incr i}
  set Path "selbox_window$i"
  toplevel .$Path
 
  set VAR($Path,Parent) .
  set VAR($Path,Title) [msgcat::mc "Files / directories selector"] 
  set VAR($Path,Type) "Dirs_Files"
  set VAR($Path,IniDir) "$env(HOME)"
  set VAR($Path,IniFile) ""
  set VAR($Path,Multiple) 0
  set VAR($Path,PreLoad) ""
  set VAR($Path,FileType) ""
  set VAR($Path,OnlyFileType) 0
  set VAR($Path,HiddenFiles) 0
  set VAR($Path,HiddenDirs) 0
  set VAR($Path,Permission) "readable"
  set VAR($Path,BalloonHelp) 0
  
  for {set i 0} {$i < [llength $args]} {incr i 2} \
  {
    set Arg [lindex $args $i]
    switch -glob -- $Arg \
    {
      -parent        {set VAR($Path,Parent) [lindex $args [expr {$i+1}]]} \
      -title            {set VAR($Path,Title) [lindex $args [expr {$i+1}]]} \
      -type           {set VAR($Path,Type) [lindex $args [expr {$i+1}]]} \
      -initialdir      {set VAR($Path,IniDir) [lindex $args [expr {$i+1}]]} \
      -initialfile     {set VAR($Path,IniFile) [lindex $args [expr {$i+1}]]} \
      -multiple     {set VAR($Path,Multiple) [lindex $args [expr {$i+1}]]} \
      -preload       {set VAR($Path,PreLoad) [lindex $args [expr {$i+1}]]} \
      -filetype      {set VAR($Path,FileType) [lindex $args [expr {$i+1}]]} \
      -only_filetype        {set VAR($Path,OnlyFileType) [lindex $args [expr {$i+1}]]} \
      -view_hidden_files {set VAR($Path,HiddenFiles) [lindex $args [expr {$i+1}]]} \
      -view_hidden_dirs  {set VAR($Path,HiddenDirs) [lindex $args [expr {$i+1}]]} \
      -permission  {set VAR($Path,Permission) [lindex $args [expr {$i+1}]]} \
      -balloonhelp {set VAR($Path,BalloonHelp) [lindex $args [expr {$i+1}]]} \
      --                  {break} \
      *                   {puts "Unknown option: $Arg"; exit 1}
    }
  }
 
  # if no parent window set -> we set it to "."
  if {![winfo exists $VAR($Path,Parent)]} \
  {
    set VAR($Path,Parent) "."
  } 
  
  # if the type was set wrong we set it to Dirs
  if {($VAR($Path,Type) ne "Dirs") && ($VAR($Path,Type) ne "Files") && ($VAR($Path,Type) ne "Dirs_Files")} \
  {
    set VAR($Path,Type) "Dirs"
  }
  
  # this stops the event loop
  set VAR($Path,Stop) 0
  
  
  ::FR_selectBox::GUI $Path
 
  ::FR_selectBox::FirstStart $Path
 
  ::FR_general::CenterWindow .$Path $VAR($Path,Parent) 1 450 300
 
  wm deiconify .$Path
 
  wm protocol .$Path WM_DELETE_WINDOW [list ::FR_selectBox::ButtonAction $Path 0]
 
  vwait ::FR_selectBox::VAR($Path,Stop)
  return $VAR($Path,Return) 
}
 
  
#---we create all widgets
proc ::FR_selectBox::GUI {Path} \
{
  variable VAR 

  wm title .$Path $VAR($Path,Title)
  wm withdraw .$Path

  labelframe .$Path.frame1 -relief groove -text [msgcat::mc "Chosen items"]
  labelframe .$Path.frame2 -relief groove -text [msgcat::mc "Actions"]
  frame .$Path.frame3 -relief flat

  pack  .$Path.frame3 -side bottom -fill x -padx 5 -pady 5
  pack  .$Path.frame1 -side left -fill both -expand yes -padx 5 -pady 5
  pack  .$Path.frame2 -side right -fill both -padx 5 -pady 5
  
  # We set a variable for each frame - so we can change the order of the frames later easier 
  set VAR($Path,listframe) "$Path.frame1"
  set VAR($Path,buttonframe) "$Path.frame2"
  set VAR($Path,bottomframe) "$Path.frame3"
 
  listbox .$VAR($Path,listframe).itemlist -background white -activestyle none -selectmode extended -yscrollcommand [list .$VAR($Path,listframe).itemlist_sbv set] -xscrollcommand [list .$VAR($Path,listframe).itemlist_sbh set]
  scrollbar .$VAR($Path,listframe).itemlist_sbv -orient vertical -command [list .$VAR($Path,listframe).itemlist yview]
  scrollbar .$VAR($Path,listframe).itemlist_sbh -orient horizontal -command [list .$VAR($Path,listframe).itemlist xview]
  
  set VAR($Path,ListBoxPath) $VAR($Path,listframe).itemlist
  
  grid .$VAR($Path,listframe).itemlist -row 0 -column 0 -sticky nsew -padx {5 0} -pady {5 0}
  grid .$VAR($Path,listframe).itemlist_sbv -row 0 -column 1 -sticky ns -padx {0 5} -pady {5 0}
  grid .$VAR($Path,listframe).itemlist_sbh -row 1 -column 0 -sticky ew -padx {5 0} -pady {0 5}
 
  grid columnconfigure .$VAR($Path,listframe) 0 -weight 1
  grid rowconfigure .$VAR($Path,listframe) 0 -weight 1 
  
  set ButtonText [::FR_general::ButtonsWithSameWidth [list [msgcat::mc "Add file(s)"] [msgcat::mc "Add directories"] [msgcat::mc "Delete item"]]]
  button .$VAR($Path,buttonframe).files -text [lindex $ButtonText 0] -state normal -width [lindex $ButtonText 3] -command [list ::FR_selectBox::AddItems $Path Files]
  button .$VAR($Path,buttonframe).dirs -text [lindex $ButtonText 1] -state normal -width [lindex $ButtonText 3] -command [list ::FR_selectBox::AddItems $Path Dirs]
  button .$VAR($Path,buttonframe).del_item -text [lindex $ButtonText 2] -state normal -width [lindex $ButtonText 3] -command [list ::FR_selectBox::DeleteItems $Path]

  pack .$VAR($Path,buttonframe).files -side top -padx 5 -pady 5
  pack .$VAR($Path,buttonframe).dirs -side top -padx 5 -pady 0
  pack .$VAR($Path,buttonframe).del_item -side top -padx 5 -pady 5
      
  # the bottom frame
  set ButtonText [::FR_general::ButtonsWithSameWidth [list [msgcat::mc "Ok"] [msgcat::mc "Cancel"]]]
  
  button .$VAR($Path,bottomframe).okay -text [lindex $ButtonText 0] -state normal -width [lindex $ButtonText 2] -command [list ::FR_selectBox::ButtonAction $Path 1]
  button .$VAR($Path,bottomframe).cancel -text [lindex $ButtonText 1] -state normal -width [lindex $ButtonText 2] -command [list ::FR_selectBox::ButtonAction $Path 0]
  
  pack .$VAR($Path,bottomframe).okay -side left -padx {5 0} -pady 5 -expand yes -fill x
  pack .$VAR($Path,bottomframe).cancel -side left -padx 5 -pady 5 -expand yes -fill x
  
  bind .$Path <KeyPress-Escape> [list .$VAR($Path,bottomframe).cancel invoke]
  bind .$Path <KeyPress-Return> [list .$VAR($Path,bottomframe).okay invoke]
}
         

# the functions for the ok/cancel buttons
proc ::FR_selectBox::ButtonAction {Path ReturnValue} \
{
  variable VAR
  
  if { $ReturnValue } \
  {
    set VAR($Path,Return) [.$VAR($Path,ListBoxPath) get 0 end]
  } else \
  {
    set VAR($Path,Return) ""
  }
 
  focus $VAR($Path,Parent)
  destroy .$Path
  set VAR($Path,Stop) 1
}


# this opens a dir or file select box and copys the selection to the listbox
proc ::FR_selectBox::AddItems {Path Type} \
{
  variable VAR
  
  set FileBox 0
  
  if {$Type eq "Files"} \
  {
    set Tmp [::FR_fileBox::Create -initialdir $VAR($Path,IniDir) -initialfile $VAR($Path,IniFile) -multiple $VAR($Path,Multiple) \
    -filetype $VAR($Path,FileType) -only_filetype $VAR($Path,OnlyFileType) -view_hidden_files $VAR($Path,HiddenFiles) \
    -view_hidden_dirs $VAR($Path,HiddenDirs) -parent .$Path -permission $VAR($Path,Permission) -balloonhelp $VAR($Path,BalloonHelp)]
    
    set FileBox 1
  } \
  elseif {$Type eq "Dirs"} \
  {
    set Tmp [::FR_dirBox::Create -initialdir $VAR($Path,IniDir) -multiple $VAR($Path,Multiple) \
    -view_hidden_dirs $VAR($Path,HiddenDirs) -parent .$Path -permission $VAR($Path,Permission) -balloonhelp $VAR($Path,BalloonHelp)]
  }
  
  if {[llength $Tmp] == 0} \
  {
    return 1
  }
  
  foreach Arg $Tmp \
  {
    .$VAR($Path,ListBoxPath) insert end $Arg
   
    if {$FileBox} \
    {
      .$VAR($Path,ListBoxPath) itemconfigure end -foreground blue
    }
  }
  
  return 0
}


# deletes one or more item(s) from the listbox
proc ::FR_selectBox::DeleteItems {Path} \
{
  variable VAR
  
  set Index [.$VAR($Path,ListBoxPath) curselection]
  
  if {$Index eq ""} \
  {
    return 1
  }
  
  set Index [lsort -decreasing $Index]
  
  foreach Arg $Index \
  {
    .$VAR($Path,ListBoxPath) delete $Arg
  }
  
  return 0
}
                                                                                                  

# first we make some presettings
proc ::FR_selectBox::FirstStart {Path} \
{
  variable VAR
  
  if {$VAR($Path,Type) eq "Dirs"} \
  {
    .$VAR($Path,buttonframe).files configure -state disable
  } \
  elseif {$VAR($Path,Type) eq "Files"} \
  {
    .$VAR($Path,buttonframe).dirs configure -state disable
  }
  
  # we preload the overtaken files / dirs into the chosen box
  if {[llength $VAR($Path,PreLoad)] > 0} \
  {
    foreach Arg $VAR($Path,PreLoad) \
    {
      if {[file exists $Arg]} \
      {
        .$VAR($Path,ListBoxPath) insert end $Arg
   
        if {[file isfile $Arg]} \
        {
          .$VAR($Path,ListBoxPath) itemconfigure end -foreground blue
        }
      }
    }
  }
  
  return 0
}


#puts [::FR_selectBox::Create -type "Files" -multiple 1]
